@@ -1,6 +1,7 @@
 module Happstack.Server.Internal.Multipart where
 
 import           Control.Monad                   (MonadPlus(mplus))
+import           Data.ByteString.Base64.Lazy
 import qualified Data.ByteString.Lazy.Char8      as L
 import           Data.ByteString.Lazy.Internal   (ByteString(Chunk, Empty))
 import qualified Data.ByteString.Lazy.UTF8       as LU
@@ -9,9 +10,7 @@ import           Data.Maybe                      (fromMaybe)
 import           Data.Int                        (Int64)
 import           Text.ParserCombinators.Parsec   (parse)
 import           Happstack.Server.Internal.Types (Input(..))
-import           Happstack.Server.Internal.RFC822Headers 
-                                                  ( ContentType(..), ContentDisposition(..), Header
-                                                  , getContentDisposition, getContentType, pHeaders)
+import           Happstack.Server.Internal.RFC822Headers
 import           System.IO                        (Handle, hClose, openBinaryTempFile)
 
 -- | similar to the normal 'span' function, except the predicate gets the whole rest of the lazy bytestring, not just one character.
@@ -21,7 +20,7 @@ spanS :: (L.ByteString -> Bool) -> L.ByteString -> (L.ByteString, L.ByteString)
 spanS f cs0 = spanS' 0 cs0
   where spanS' _ Empty = (Empty, Empty)
         spanS' n bs@(Chunk c cs)
-            | n >= S.length c = 
+            | n >= S.length c =
                 let (x, y) = spanS' 0 cs
                 in (Chunk c x, y)
             | not (f (Chunk (S.drop n c) cs)) = L.splitAt (fromIntegral n) bs
@@ -47,7 +46,7 @@ blankLine = L.pack "\r\n\r\n"
 
 dropWhileS :: (L.ByteString -> Bool) -> L.ByteString -> L.ByteString
 dropWhileS f cs0 = dropWhile' cs0
-    where dropWhile' bs 
+    where dropWhile' bs
               | L.null bs  = bs
               | f bs       = dropWhile' (L.drop 1 bs)
               | otherwise  = bs
@@ -55,29 +54,29 @@ dropWhileS f cs0 = dropWhile' cs0
 data BodyPart = BodyPart L.ByteString L.ByteString  -- ^ headers body
     deriving (Eq, Ord, Read, Show)
 
-data Work 
+data Work
     = BodyWork ContentType [(String, String)] L.ByteString
-    | HeaderWork L.ByteString 
+    | HeaderWork L.ByteString
 
 type InputWorker = Work -> IO InputIter
 
-data InputIter 
+data InputIter
     = Failed (Maybe (String, Input)) String
     | BodyResult (String, Input) InputWorker
     | HeaderResult [Header] InputWorker
 
-type FileSaver = FilePath 		-- ^ tempdir
-		-> Int64 		-- ^ quota
-		-> FilePath 		-- ^ filename of field
-		-> L.ByteString 	-- ^ content to save
-		-> IO (Bool, Int64 , FilePath)	-- ^ truncated?, saved bytes, saved filename
+type FileSaver = FilePath               -- ^ tempdir
+                -> Int64                -- ^ quota
+                -> FilePath             -- ^ filename of field
+                -> L.ByteString         -- ^ content to save
+                -> IO (Bool, Int64 , FilePath)  -- ^ truncated?, saved bytes, saved filename
 
 defaultFileSaver :: FilePath -> Int64 -> FilePath -> ByteString -> IO (Bool, Int64, FilePath)
 defaultFileSaver tmpDir diskQuota filename b =
     do (fn, h) <- openBinaryTempFile tmpDir filename
        (trunc, len) <- hPutLimit diskQuota h b
        hClose h
-       return (trunc, len, fn) 
+       return (trunc, len, fn)
 
 defaultInputIter :: FileSaver -> FilePath -> Int64 -> Int64 -> Int64 -> Int64 -> Int64 -> Int64 -> Work -> IO InputIter
 defaultInputIter fileSaver tmpDir diskCount ramCount headerCount maxDisk maxRAM maxHeader (BodyWork ctype ps b)
@@ -85,7 +84,7 @@ defaultInputIter fileSaver tmpDir diskCount ramCount headerCount maxDisk maxRAM
     | ramCount  > maxRAM  = return $ Failed Nothing ("ramCount ("  ++ show ramCount  ++ ") is greater than maxRAM ("  ++ show maxRAM   ++ ")")
     | otherwise =
         case lookup "filename" ps of
-          Nothing -> 
+          Nothing ->
               let (b',rest) = L.splitAt (maxRAM - ramCount) b
                   input = (fromMaybe "" $ lookup "name" ps
                           , Input { inputValue       = (Right b')
@@ -141,14 +140,32 @@ bodyPartToInput inputWorker (BodyPart rawHS b) =
          (HeaderResult hs cont) ->
           let ctype = fromMaybe defaultInputType (getContentType hs) in
           case getContentDisposition hs of
-              Just (ContentDisposition "form-data" ps) ->
-                  cont (BodyWork ctype ps b)
-
+              Just (ContentDisposition "form-data" ps) -> do
+                  let eb' = case getContentTransferEncoding hs of
+                            Nothing -> Right b
+                            Just (ContentTransferEncoding "7bit") ->
+                                -- We don't bother checking that the data
+                                -- really is 7bit-only
+                                Right b
+                            Just (ContentTransferEncoding "8bit") ->
+                                Right b
+                            Just (ContentTransferEncoding "binary") ->
+                                Right b
+                            Just (ContentTransferEncoding "base64") ->
+                                Right $ decodeLenient b
+                            -- TODO: Support quoted-printable
+                            Just cte ->
+                                Left ("Bad content-transfer-encoding: " ++ show cte)
+                  case eb' of
+                      Right b' ->
+                          cont (BodyWork ctype ps b')
+                      Left err ->
+                          return $ Failed Nothing err
               cd -> return $ Failed Nothing ("Expected content-disposition: form-data but got " ++ show cd)
          (BodyResult {}) -> return $ Failed Nothing "bodyPartToInput: Got unexpected BodyResult."
 
 bodyPartsToInputs :: InputWorker -> [BodyPart] -> IO ([(String,Input)], Maybe String)
-bodyPartsToInputs _ [] = 
+bodyPartsToInputs _ [] =
     return ([], Nothing)
 bodyPartsToInputs inputWorker (b:bs) =
     do r <- bodyPartToInput inputWorker b
@@ -246,7 +263,7 @@ splitAtEmptyLine s =
       (before, after) | L.null after -> Nothing
                       | otherwise    -> Just (L.append before crlf, L.drop 4 after)
 {-# INLINE splitAtEmptyLine #-}
-      
+
 -- | Split a string at the first CRLF. The CRLF is not included
 --   in any of the returned strings.
 splitAtCRLF :: ByteString -- ^ String to split.
