@@ -13,9 +13,10 @@ module Happstack.Server.Internal.Types
      redirect, -- redirect_, redirect', redirect'_,
      isHTTP1_0, isHTTP1_1,
      RsFlags(..), nullRsFlags, contentLength, chunked, noContentLength,
-     HttpVersion(..), Length(..), Method(..), Headers, continueHTTP,
+     HttpVersion(..), Length(..), Method(..), canHaveBody, Headers, continueHTTP,
      Host, ContentType(..),
-     readDec', fromReadS, readM, FromReqURI(..)
+     readDec', fromReadS, readM, FromReqURI(..),
+     showRsValidator
     ) where
 
 import Control.Monad.Error (Error(strMsg))
@@ -44,7 +45,6 @@ import Happstack.Server.Internal.Cookie
 import Happstack.Server.Internal.LogFormat (formatRequestCombined)
 import Numeric (readDec, readSigned)
 import System.Log.Logger (Priority(..), logM)
-import Text.Show.Functions ()
 
 -- | HTTP version
 data HttpVersion = HttpVersion Int Int
@@ -132,8 +132,20 @@ logMAccess host user time requestLine responseCode size referer userAgent =
     logM "Happstack.Server.AccessLog.Combined" INFO $ formatRequestCombined host user time requestLine responseCode size referer userAgent
 
 -- | HTTP request method
-data Method  = GET | HEAD | POST | PUT | DELETE | TRACE | OPTIONS | CONNECT
-               deriving(Show,Read,Eq,Ord,Typeable,Data)
+data Method = GET | HEAD | POST | PUT | DELETE | TRACE | OPTIONS | CONNECT | PATCH | EXTENSION ByteString
+    deriving (Show,Read,Eq,Ord,Typeable,Data)
+
+-- | Does the method support a message body?
+--
+-- For extension methods, we assume yes.
+canHaveBody :: Method
+            -> Bool
+canHaveBody POST          = True
+canHaveBody PUT           = True
+canHaveBody PATCH         = True
+canHaveBody DELETE        = True
+canHaveBody (EXTENSION _) = True
+canHaveBody _             = False
 
 -- | an HTTP header
 data HeaderPair = HeaderPair
@@ -211,22 +223,25 @@ data Response
 
 instance Show Response where
     showsPrec _ res@Response{}  =
-        showString   "================== Response ================" .
-        showString "\nrsCode      = " . shows      (rsCode res)     .
-        showString "\nrsHeaders   = " . shows      (rsHeaders res)  .
-        showString "\nrsFlags     = " . shows      (rsFlags res)    .
-        showString "\nrsBody      = " . shows      (rsBody res)     .
-        showString "\nrsValidator = " . shows      (rsValidator res)
+        showString   "================== Response ================"                    .
+        showString "\nrsCode      = " . shows      (rsCode res)                        .
+        showString "\nrsHeaders   = " . shows      (rsHeaders res)                     .
+        showString "\nrsFlags     = " . shows      (rsFlags res)                       .
+        showString "\nrsBody      = " . shows      (rsBody res)                        .
+        showString "\nrsValidator = " . shows      (showRsValidator (rsValidator res))
     showsPrec _ res@SendFile{}  =
-        showString   "================== Response ================" .
-        showString "\nrsCode      = " . shows      (rsCode res)     .
-        showString "\nrsHeaders   = " . shows      (rsHeaders res)  .
-        showString "\nrsFlags     = " . shows      (rsFlags res)    .
-        showString "\nrsValidator = " . shows      (rsValidator res).
-        showString "\nsfFilePath  = " . shows      (sfFilePath res) .
-        showString "\nsfOffset    = " . shows      (sfOffset res)   .
+        showString   "================== Response ================"                    .
+        showString "\nrsCode      = " . shows      (rsCode res)                        .
+        showString "\nrsHeaders   = " . shows      (rsHeaders res)                     .
+        showString "\nrsFlags     = " . shows      (rsFlags res)                       .
+        showString "\nrsValidator = " . shows      (showRsValidator (rsValidator res)) .
+        showString "\nsfFilePath  = " . shows      (sfFilePath res)                    .
+        showString "\nsfOffset    = " . shows      (sfOffset res)                      .
         showString "\nsfCount     = " . shows      (sfCount res)
 
+showRsValidator :: Maybe (Response -> IO Response) -> String
+showRsValidator = maybe "Nothing" (const "Just <function>")
+
 -- what should the status code be ?
 instance Error Response where
   strMsg str =
